@@ -13,9 +13,8 @@ namespace internal {
 // Generic range histograms.
 // HR(name, caption, min, max, num_buckets)
 #define HISTOGRAM_RANGE_LIST(HR)                                               \
-  HR(code_cache_reject_reason, V8.CodeCacheRejectReason, 1, 6, 6)              \
+  HR(code_cache_reject_reason, V8.CodeCacheRejectReason, 1, 9, 9)              \
   HR(errors_thrown_per_context, V8.ErrorsThrownPerContext, 0, 200, 20)         \
-  HR(debug_feature_usage, V8.DebugFeatureUsage, 1, 7, 7)                       \
   HR(incremental_marking_reason, V8.GCIncrementalMarkingReason, 0,             \
      kGarbageCollectionReasonMaxValue, kGarbageCollectionReasonMaxValue + 1)   \
   HR(incremental_marking_sum, V8.GCIncrementalMarkingSum, 0, 10000, 101)       \
@@ -47,14 +46,6 @@ namespace internal {
      100 * KB, GB, 51)                                                         \
   HR(wasm_asm_module_size_bytes, V8.WasmModuleSizeBytes.asm, 1, GB, 51)        \
   HR(wasm_wasm_module_size_bytes, V8.WasmModuleSizeBytes.wasm, 1, GB, 51)      \
-  HR(wasm_asm_min_mem_pages_count, V8.WasmMinMemPagesCount.asm, 1, 2 << 16,    \
-     51)                                                                       \
-  HR(wasm_wasm_min_mem_pages_count, V8.WasmMinMemPagesCount.wasm, 1, 2 << 16,  \
-     51)                                                                       \
-  HR(wasm_wasm_max_mem_pages_count, V8.WasmMaxMemPagesCount.wasm, 1, 2 << 16,  \
-     51)                                                                       \
-  HR(wasm_compile_function_peak_memory_bytes,                                  \
-     V8.WasmCompileFunctionPeakMemoryBytes, 1, GB, 51)                         \
   HR(wasm_compile_huge_function_peak_memory_bytes,                             \
      V8.WasmCompileHugeFunctionPeakMemoryBytes, 1, GB, 51)                     \
   HR(asm_module_size_bytes, V8.AsmModuleSizeBytes, 1, GB, 51)                  \
@@ -62,7 +53,10 @@ namespace internal {
      21)                                                                       \
   HR(wasm_memory_allocation_result, V8.WasmMemoryAllocationResult, 0, 3, 4)    \
   /* Committed code size per module, collected on GC. */                       \
+  /* Older histogram, in MiB (0..1024MB). */                                   \
   HR(wasm_module_code_size_mb, V8.WasmModuleCodeSizeMiB, 0, 1024, 64)          \
+  /* Newer histogram, in KiB (0..100MB). */                                    \
+  HR(wasm_module_code_size_kb, V8.WasmModuleCodeSizeKiB, 0, 1024 * 100, 101)   \
   /* Percent of freed code size per module, collected on GC. */                \
   HR(wasm_module_freed_code_size_percent, V8.WasmModuleCodeSizePercentFreed,   \
      0, 100, 32)                                                               \
@@ -80,12 +74,6 @@ namespace internal {
   /* Support for PKEYs/PKU by testing result of pkey_alloc(). */               \
   HR(wasm_memory_protection_keys_support, V8.WasmMemoryProtectionKeysSupport,  \
      0, 1, 2)                                                                  \
-  /* Number of thrown exceptions per isolate. */                               \
-  HR(wasm_throw_count, V8.WasmThrowCount, 0, 100000, 30)                       \
-  /* Number of rethrown exceptions per isolate. */                             \
-  HR(wasm_rethrow_count, V8.WasmReThrowCount, 0, 100000, 30)                   \
-  /* Number of caught exceptions per isolate. */                               \
-  HR(wasm_catch_count, V8.WasmCatchCount, 0, 100000, 30)                       \
   /* Ticks observed in a single Turbofan compilation, in 1K. */                \
   HR(turbofan_ticks, V8.TurboFan1KTicks, 0, 100000, 200)                       \
   /* Backtracks observed in a single regexp interpreter execution. */          \
@@ -101,6 +89,10 @@ namespace internal {
   /* Counted after sweeping the table at the end of mark-compact GC. */        \
   HR(external_pointers_count, V8.SandboxedExternalPointersCount, 0,            \
      kMaxExternalPointers, 101)                                                \
+  HR(code_pointers_count, V8.SandboxedCodePointersCount, 0, kMaxCodePointers,  \
+     101)                                                                      \
+  HR(trusted_pointers_count, V8.SandboxedTrustedPointersCount, 0,              \
+     kMaxTrustedPointers, 101)                                                 \
   HR(wasm_num_lazy_compilations_5sec, V8.WasmNumLazyCompilations5Sec, 0,       \
      200000, 50)                                                               \
   HR(wasm_num_lazy_compilations_20sec, V8.WasmNumLazyCompilations20Sec, 0,     \
@@ -114,7 +106,8 @@ namespace internal {
   /* ExternalPointerTable::TableCompactionOutcome enum for more details. */    \
   HR(external_pointer_table_compaction_outcome,                                \
      V8.ExternalPointerTableCompactionOutcome, 0, 2, 3)                        \
-  HR(wasm_compilation_method, V8.WasmCompilationMethod, 0, 4, 5)
+  HR(wasm_compilation_method, V8.WasmCompilationMethod, 0, 4, 5)               \
+  HR(asmjs_instantiate_result, V8.AsmjsInstantiateResult, 0, 1, 2)
 
 // Like TIMED_HISTOGRAM_LIST, but allows the use of NestedTimedHistogramScope.
 // HT(name, caption, max, unit)
@@ -138,6 +131,8 @@ namespace internal {
   HT(compile_deserialize, V8.CompileDeserializeMicroSeconds, 1000000,         \
      MICROSECOND)                                                             \
   /* Snapshot. */                                                             \
+  HT(snapshot_decompress, V8.SnapshotDecompressMicroSeconds, 1000000,         \
+     MICROSECOND)                                                             \
   HT(snapshot_deserialize_rospace, V8.SnapshotDeserializeRoSpaceMicroSeconds, \
      1000000, MICROSECOND)                                                    \
   HT(snapshot_deserialize_isolate, V8.SnapshotDeserializeIsolateMicroSeconds, \
@@ -155,20 +150,66 @@ namespace internal {
 // Timer histograms, thread safe: HT(name, caption, max, unit)
 #define TIMED_HISTOGRAM_LIST(HT)                                               \
   /* Garbage collection timers. */                                             \
-  HT(gc_compactor, V8.GCCompactor, 10000, MILLISECOND)                         \
-  HT(gc_compactor_background, V8.GCCompactorBackground, 10000, MILLISECOND)    \
-  HT(gc_compactor_foreground, V8.GCCompactorForeground, 10000, MILLISECOND)    \
-  HT(gc_finalize, V8.GCFinalizeMC, 10000, MILLISECOND)                         \
-  HT(gc_finalize_background, V8.GCFinalizeMCBackground, 10000, MILLISECOND)    \
-  HT(gc_finalize_foreground, V8.GCFinalizeMCForeground, 10000, MILLISECOND)    \
-  HT(gc_finalize_measure_memory, V8.GCFinalizeMCMeasureMemory, 10000,          \
+  HT(gc_finalize_incremental_regular,                                          \
+     V8.GC.Event.MainThread.Full.Finalize.Incremental.Regular, 10000,          \
      MILLISECOND)                                                              \
-  HT(gc_finalize_reduce_memory, V8.GCFinalizeMCReduceMemory, 10000,            \
+  HT(gc_finalize_incremental_regular_foreground,                               \
+     V8.GC.Event.MainThread.Full.Finalize.Incremental.Regular.Foreground,      \
+     10000, MILLISECOND)                                                       \
+  HT(gc_finalize_incremental_regular_background,                               \
+     V8.GC.Event.MainThread.Full.Finalize.Incremental.Regular.Background,      \
+     10000, MILLISECOND)                                                       \
+  HT(gc_finalize_incremental_memory_reducing,                                  \
+     V8.GC.Event.MainThread.Full.Finalize.Incremental.ReduceMemory, 10000,     \
      MILLISECOND)                                                              \
-  HT(gc_finalize_reduce_memory_background,                                     \
-     V8.GCFinalizeMCReduceMemoryBackground, 10000, MILLISECOND)                \
-  HT(gc_finalize_reduce_memory_foreground,                                     \
-     V8.GCFinalizeMCReduceMemoryForeground, 10000, MILLISECOND)                \
+  HT(gc_finalize_incremental_memory_reducing_foreground,                       \
+     V8.GC.Event.MainThread.Full.Finalize.Incremental.ReduceMemory.Foreground, \
+     10000, MILLISECOND)                                                       \
+  HT(gc_finalize_incremental_memory_reducing_background,                       \
+     V8.GC.Event.MainThread.Full.Finalize.Incremental.ReduceMemory.Background, \
+     10000, MILLISECOND)                                                       \
+  HT(gc_finalize_incremental_memory_measure,                                   \
+     V8.GC.Event.MainThread.Full.Finalize.Incremental.MeasureMemory, 10000,    \
+     MILLISECOND)                                                              \
+  HT(gc_finalize_incremental_memory_measure_foreground,                        \
+     V8.GC.Event.MainThread.Full.Finalize.Incremental.MeasureMemory            \
+         .Foreground,                                                          \
+     10000, MILLISECOND)                                                       \
+  HT(gc_finalize_incremental_memory_measure_background,                        \
+     V8.GC.Event.MainThread.Full.Finalize.Incremental.MeasureMemory            \
+         .Background,                                                          \
+     10000, MILLISECOND)                                                       \
+  HT(gc_finalize_non_incremental_regular,                                      \
+     V8.GC.Event.MainThread.Full.Finalize.NonIncremental.Regular, 10000,       \
+     MILLISECOND)                                                              \
+  HT(gc_finalize_non_incremental_regular_foreground,                           \
+     V8.GC.Event.MainThread.Full.Finalize.NonIncremental.Regular.Foreground,   \
+     10000, MILLISECOND)                                                       \
+  HT(gc_finalize_non_incremental_regular_background,                           \
+     V8.GC.Event.MainThread.Full.Finalize.NonIncremental.Regular.Background,   \
+     10000, MILLISECOND)                                                       \
+  HT(gc_finalize_non_incremental_memory_reducing,                              \
+     V8.GC.Event.MainThread.Full.Finalize.NonIncremental.ReduceMemory, 10000,  \
+     MILLISECOND)                                                              \
+  HT(gc_finalize_non_incremental_memory_reducing_foreground,                   \
+     V8.GC.Event.MainThread.Full.Finalize.NonIncremental.ReduceMemory          \
+         .Foreground,                                                          \
+     10000, MILLISECOND)                                                       \
+  HT(gc_finalize_non_incremental_memory_reducing_background,                   \
+     V8.GC.Event.MainThread.Full.Finalize.NonIncremental.ReduceMemory          \
+         .Background,                                                          \
+     10000, MILLISECOND)                                                       \
+  HT(gc_finalize_non_incremental_memory_measure,                               \
+     V8.GC.Event.MainThread.Full.Finalize.NonIncremental.MeasureMemory, 10000, \
+     MILLISECOND)                                                              \
+  HT(gc_finalize_non_incremental_memory_measure_foreground,                    \
+     V8.GC.Event.MainThread.Full.Finalize.NonIncremental.MeasureMemory         \
+         .Foreground,                                                          \
+     10000, MILLISECOND)                                                       \
+  HT(gc_finalize_non_incremental_memory_measure_background,                    \
+     V8.GC.Event.MainThread.Full.Finalize.NonIncremental.MeasureMemory         \
+         .Background,                                                          \
+     10000, MILLISECOND)                                                       \
   HT(measure_memory_delay_ms, V8.MeasureMemoryDelayMilliseconds, 100000,       \
      MILLISECOND)                                                              \
   HT(gc_time_to_global_safepoint, V8.GC.TimeToGlobalSafepoint, 10000000,       \
@@ -220,8 +261,6 @@ namespace internal {
      V8.WasmFinishModuleStreamingMicroSeconds, 100000000, MICROSECOND)         \
   HT(wasm_deserialization_time, V8.WasmDeserializationTimeMilliSeconds, 10000, \
      MILLISECOND)                                                              \
-  HT(wasm_tier_up_module_time, V8.WasmTierUpModuleMicroSeconds, 100000000,     \
-     MICROSECOND)                                                              \
   HT(wasm_compile_asm_function_time, V8.WasmCompileFunctionMicroSeconds.asm,   \
      1000000, MICROSECOND)                                                     \
   HT(wasm_compile_wasm_function_time, V8.WasmCompileFunctionMicroSeconds.wasm, \
@@ -232,12 +271,6 @@ namespace internal {
      V8.WasmInstantiateModuleMicroSeconds.wasm, 10000000, MICROSECOND)         \
   HT(wasm_instantiate_asm_module_time,                                         \
      V8.WasmInstantiateModuleMicroSeconds.asm, 10000000, MICROSECOND)          \
-  HT(wasm_time_between_throws, V8.WasmTimeBetweenThrowsMilliseconds, 1000,     \
-     MILLISECOND)                                                              \
-  HT(wasm_time_between_rethrows, V8.WasmTimeBetweenRethrowsMilliseconds, 1000, \
-     MILLISECOND)                                                              \
-  HT(wasm_time_between_catch, V8.WasmTimeBetweenCatchMilliseconds, 1000,       \
-     MILLISECOND)                                                              \
   HT(wasm_lazy_compile_time, V8.WasmLazyCompileTimeMicroSeconds, 100000000,    \
      MICROSECOND)                                                              \
   HT(wasm_compile_after_deserialize,                                           \
@@ -266,6 +299,9 @@ namespace internal {
      V8.CompileScriptMicroSeconds.BackgroundThread, 1000000, MICROSECOND)      \
   HT(compile_function_on_background,                                           \
      V8.CompileFunctionMicroSeconds.BackgroundThread, 1000000, MICROSECOND)    \
+  HT(deserialize_script_on_background,                                         \
+     V8.CompileScriptMicroSeconds.ConsumeCache.BackgroundThread, 1000000,      \
+     MICROSECOND)                                                              \
   HT(wasm_max_lazy_compilation_time_5sec,                                      \
      V8.WasmMaxLazyCompilationTime5SecMilliSeconds, 5000, MILLISECOND)         \
   HT(wasm_max_lazy_compilation_time_20sec,                                     \
@@ -323,6 +359,7 @@ namespace internal {
      V8.GCCompactorCausedByOldspaceExhaustion)                                 \
   SC(enum_cache_hits, V8.EnumCacheHits)                                        \
   SC(enum_cache_misses, V8.EnumCacheMisses)                                    \
+  SC(maps_created, V8.MapsCreated)                                             \
   SC(megamorphic_stub_cache_updates, V8.MegamorphicStubCacheUpdates)           \
   SC(regexp_entry_runtime, V8.RegExpEntryRuntime)                              \
   SC(stack_interrupts, V8.StackInterrupts)                                     \
