@@ -9,16 +9,17 @@ const {
   FunctionPrototypeApply,
   FunctionPrototypeBind,
   FunctionPrototypeToString,
-  globalThis,
   NumberIsNaN,
-  ObjectDefineProperty,
   ObjectDefineProperties,
+  ObjectDefineProperty,
   ObjectGetOwnPropertyDescriptor,
   ObjectGetOwnPropertyDescriptors,
   Promise,
   Symbol,
   SymbolAsyncIterator,
+  globalThis,
 } = primordials;
+
 const {
   validateAbortSignal,
   validateNumber,
@@ -31,7 +32,10 @@ const {
 } = require('internal/util');
 const {
   AbortError,
-  codes: { ERR_INVALID_STATE, ERR_INVALID_ARG_VALUE },
+  codes: {
+    ERR_INVALID_ARG_VALUE,
+    ERR_INVALID_STATE,
+  },
 } = require('internal/errors');
 
 const { TIMEOUT_MAX } = require('internal/timers');
@@ -398,7 +402,7 @@ class MockTimers {
       return FunctionPrototypeToString(MockDate[kMock].#nativeDateDescriptor.value);
     };
 
-    // We need to polute the prototype of this
+    // We need to pollute the prototype of this
     ObjectDefineProperties(MockDate, {
       __proto__: null,
       [kMock]: {
@@ -718,13 +722,9 @@ class MockTimers {
       throw new ERR_INVALID_ARG_VALUE('now', internalOptions.now, `epoch must be a positive integer received ${internalOptions.now}`);
     }
 
-    if (!internalOptions.now) {
-      internalOptions.now = 0;
-    }
+    internalOptions.now ||= 0;
 
-    if (!internalOptions.apis) {
-      internalOptions.apis = SUPPORTED_APIS;
-    }
+    internalOptions.apis ||= SUPPORTED_APIS;
 
     validateStringArray(internalOptions.apis, 'options.apis');
     // Check that the timers passed are supported
